<?php

namespace v1\controllers;

use app\models\ObraEmpleado;
use app\models\ObraHerramienta;
use common\data\Respuesta;
use common\rest\AuthController;
use Yii;
use yii\db\Expression;

class ObraController extends AuthController {

  public $modelClass = "v1\models\Obra";

  public function actionIndex() {
    $id = trim($this->req->get("id", ""));
    $buscar = trim($this->req->get("q", ""));
    $rangoInicio = trim($this->req->get("inicio", ""));
    $rangoFin = trim($this->req->get("fin", ""));
    $rangoI = trim($this->req->get("rinicio", ""));
    $rangoF = trim($this->req->get("rfin", ""));
    $query = $this->queryInicial;

    if ($id > 0) {
      $query->andWhere(["id" => $id]);
    }

    if ($rangoInicio !== "" && $rangoFin !== "") {
      $query->andWhere(["between", "[[fechaInicio]]", $rangoInicio, $rangoFin]);
    }
    if ($rangoI !== "" && $rangoF !== "") {
      $query->andWhere(["between", "[[fechaFinal]]", $rangoI, $rangoF]);
    }
    if ($buscar) {

      $query->andWhere([
        "OR",
        ["ilike", "nombre", $buscar]
      ]);
    }

    return new Respuesta($query, $this->limite, $this->pagina, $this->ordenar);
  }

  public function actionGuardar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $herramienta = $this->req->getBodyParam("herramientas", []);
    $empleado = $this->req->getBodyParam("empleados", []);
    $modelo = null;

    $transaccion = \Yii::$app->db->beginTransaction();
    try {

      if ($id !== "") {
        $modelo = $this->modelClass::findOne($id);
      }
      if ($modelo === null) {
        $modelo = new $this->modelClass();
        $modelo->uuid();
        $modelo->creado = new Expression('now()');
      } else {
        $modelo->modificado = new Expression('now()');
      }
      $modelo->load($this->req->getBodyParams(), '');

      $clave = $modelo->clave;
      $existeClave = $this->modelClass::find()->where(['clave' => $clave])->andWhere(['<>', 'id', $id])->exists();

      if ($existeClave) {
        return (new Respuesta())
          ->esError()
          ->mensaje("La clave '$clave' ya existe. Por favor, ingresa una clave diferente.");
      }
      $modelo->load($this->req->getBodyParams(), '');
      if (!$modelo->save()) {
        return (new Respuesta($modelo))
          ->mensaje("Hubo un problema al guardar la obra");
      }

      ObraEmpleado::deleteAll(['id' => $modelo->id]);

      foreach ($empleado as $item) {
        if (isset($item['idEmpleado']['value']) && !empty($item['idEmpleado']['value'])) {
          $interMedia = new ObraEmpleado();
          $interMedia->id = \Yii::$app->security->generateRandomString(36);
          $interMedia->idEmpleado = $item['idEmpleado']['value'];
          $interMedia->idObra = $modelo->id;

          if (!$interMedia->save()) {
            $transaccion->rollBack();
            return (new Respuesta($interMedia))
              ->mensaje("Hubo un problema al guardar el Empleado con ID {$item['idEmpleado']['value']}");
          }
        } else {
          Yii::error('Falta el idEmpleado o está vacío en el array', ['item' => $item]);
        }
      }

      ObraHerramienta::deleteAll(['id' => $modelo->id]);

      foreach ($herramienta as $item) {
        if (isset($item['idHerramienta']['value']) && isset($item['cantidad'])) {
          $interMedia = new ObraHerramienta();
          $interMedia->id = \Yii::$app->security->generateRandomString(36);
          $interMedia->idHerramienta = $item['idHerramienta']['value'];
          $interMedia->idObra = $modelo->id;
          $interMedia->cantidad = $item['cantidad'];

          if (!$interMedia->save()) {
            $transaccion->rollBack();
            return (new Respuesta($interMedia))
              ->mensaje("Hubo un problema al guardar la Herramienta con ID {$item['idHerramienta']['value']}");
          }
        } else {
          Yii::error('Faltan datos en el array de herramientas', ['item' => $item]);
        }
      }

      $transaccion->commit();
      $modelo->refresh();
      return (new Respuesta($modelo))
        ->mensaje("Obra ha sido guardado de manera exitosa");
    } catch (\Exception $e) {
      $transaccion->rollBack();
      return (new Respuesta($modelo))
        ->esError()
        ->mensaje($e->getMessage());
    }
  }

  public function actionEliminar() {
    $id = trim($this->req->getBodyParam("id", ""));
    $modelo = null;

    if ($id !== "") {
      $modelo = $this->modelClass::findOne(["id" => $id]);
    }
    if ($modelo === null) {
      return (new Respuesta())
        ->esError()
        ->mensaje("Obra no encontrada");
    }
    $modelo->eliminado = new Expression('now()');
    if (!$modelo->save()) {
      return (new Respuesta($modelo))
        ->mensaje("No se pudo eliminar la obra");
    }

    return (new Respuesta())
      ->mensaje("Obra eliminada");
  }
}
